##############################################################################
#
# CutImageField - Field that cut images for Archetypes
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################
"""
$Id: __init__.py,v 0.1 2005/07/28 11:08:10 ddrillon Exp $
"""

from AccessControl import ClassSecurityInfo
from Products.Archetypes.Field import ImageField
from Products.Archetypes.Widget import ImageWidget
from Products.Archetypes.Registry import registerField

from Products.CMFCore import CMFCorePermissions
from ZODB.POSException import ConflictError

from cStringIO import StringIO


try:
    import PIL.Image
    HAS_PIL=True
except ImportError:
    # no PIL, no scaled versions!
    log("Warning: no Python Imaging Libraries (PIL) found."+\
        "Archetypes based ImageField's don't scale if neccessary.")
    HAS_PIL=False
			    
_marker=[]

class CutImageField(ImageField):
    """
    cut       -- if an Image is scaled, it can be smaller than wished,
                 since the scale takes into account the smallest dimension.
                 For example, if original_size is set to (450,250), the result image
                 will be 250 height but since the aspect ratio is kept, the image
                 probably won't be 450 weight.
                 if cut_taller_than is set, the scale will take into account the taller
                 argument (450) and cut off height if it is taller.
    """
    _properties = ImageField._properties.copy()
    _properties.update({
        'cut': None,
	})

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'rescaleOriginal')
    def rescaleOriginal(self, value, **kwargs):
        """
	rescales the original image and sets the data
        for self.original_size or self.max_size
	"""
	mimetype = kwargs.get('mimetype', 'image/png')
	if HAS_PIL:
	    if self.original_size or self.max_size :
		data = str(value.data)
		w=h=0
		cut_key=''
		if self.max_size:
		    if value.width > self.max_size[0] or \
		       value.height > self.max_size[1]:
		       factor = min(float(self.max_size[0])/float(value.width),
		                    float(self.max_size[1])/float(value.height))
		       w = int(factor*value.width)
		       h = int(factor*value.height)
		       cut_key='max_size'
		elif self.original_size:
		       w,h = self.original_size
		       cut_key='original_size'
		if w and h:
		     __traceback_info__ = (self, value, w, h)
		     if self.cut :
		         if self.cut.has_key(cut_key) and self.cut[cut_key] == 1 :
			     if value.width > w and value.height > h :
			         # processing scale
				 ratio=float(w)/float(h)
				 image_ratio = float(value.width)/float(value.height)
				 if image_ratio > ratio:
				     fvalue, format = self.scale(data,w*100,h)
				 elif image_ratio < ratio :
				     fvalue, format = self.scale(data,w,h*100)
				 # processing cut
				 fvalue, format = self.cutImage(str(fvalue.read()),w,h)
			     else :
				  fvalue, format = self.cutImage(data,w,h)
		     else :
			 fvalue, format = self.scale(data,w,h)
		     data = fvalue.read()
	return data


    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'createScales')
    def createScales(self, instance,value=_marker):
        """creates the scales and save them
        """
	print "trying scaling..."
        sizes = self.getAvailableSizes(instance)
        if not HAS_PIL or not sizes:
            return
        # get data from the original size if value is None
	img_width=0
	img_height=0
        if value is _marker:
	    img = self.getRaw(instance) 
            if not img:
                return
 	    data = str(img.data)
	    img_width=img.width
	    img_height=img.height
	else:
	    img=self.getRaw(instance)
	    test = str(img.data)
	    data = value
	    if (test==data):
	        img_width=img.width
		img_height=img.height
	if not data:
	    return
	print "scaling... "
        filename = self.getFilename(instance)
        for n, size in sizes.items():
            w, h = size
            id = self.getName() + "_" + n
            __traceback_info__ = (self, instance, id, w, h)
            try:
                if self.cut :
                    if self.cut.has_key(n) and self.cut[n] == 1 :
                        if img_width > w and img_height > h :
                            # processing scale
			    print "processing scale"
                            ratio=float(w)/float(h)
                            image_ratio = float(img_width)/float(img_height)
                            if image_ratio > ratio:
                                imgdata, format = self.scale(data,w*100,h)
                            elif image_ratio < ratio :
                                imgdata, format = self.scale(data,w,h*100)
                            # processing cut
                            imgdata, format = self.cutImage(str(imgdata.read()),w,h)
                        else :
                            imgdata, format = self.cutImage(data,w,h)
                else :
                    imgdata, format = self.scale(data,w,h)

            except ConflictError:
	                raise
            except:
                if not self.swallowResizeExceptions:
                    raise
                else:
                    log_exc()
                    # scaling failed, don't create a scaled version
                    continue
            mimetype = 'image/%s' % format.lower()
            image = self.content_class(id, self.getName(),
                                     imgdata,
                                     mimetype
                                     )
            image.filename = filename
            # manually use storage
            delattr(image, 'title')
            self.getStorage(instance).set(id, instance, image,
                                          mimetype=mimetype, filename=filename)

    security.declarePrivate('cutImage')
    def cutImage(self, data, w, h):
        """ cut image (with material from ImageTag_Hotfix)"""
        #make sure we have valid int's
        size = int(w), int(h)
        pilfilter = PIL.Image.NEAREST
        #check for the pil version and enable antialias if > 1.1.3
        if PIL.Image.VERSION >= "1.1.3":
            pilfilter = PIL.Image.ANTIALIAS
        original_file=StringIO(data)
        image = PIL.Image.open(original_file)
        old_w,old_h = image.size
        # consider image mode when scaling
        # source images can be mode '1','L,','P','RGB(A)'
        # convert to greyscale or RGBA before scaling
        # preserve palletted mode (but not pallette)
        # for palletted-only image formats, e.g. GIF
        # PNG compression is OK for RGBA thumbnails
        original_mode = image.mode
        if original_mode == '1':
            image = image.convert('L')
        elif original_mode == 'P':
            image = image.convert('RGBA')
        x=0
        if old_h > h:
            cut_h=(old_h-h)/2
            x=1
            #avoiding vertical black band
            if old_w < w :
                image2=image.crop((0,cut_h,old_w,h+cut_h))
            else :
                image2 = image.crop((0,cut_h,w,h+cut_h))
        if old_w > w:
            cut_w = (old_w - w)/2
            x=1
            #avoiding horizontal black band
            if old_h < h :
                image2=image.crop((cut_w,0,cut_w + w,old_h))
            else :
                image2=image.crop((cut_w,0,cut_w + w,h))
        if x==0:
            image.thumbnail(size, pilfilter)
        # XXX: tweak to make the unit test
        #      test_fields.ProcessingTest.test_processing_fieldset run
        format = image.format and image.format or 'PNG'
        # decided to only preserve palletted mode
        # for GIF, could also use image.format in ('GIF','PNG')
        if original_mode == 'P' and format == 'GIF':
            image = image.convert('P')
        thumbnail_file = StringIO()
        # quality parameter doesn't affect lossless formats
        if x == 1 :
            image2.save(thumbnail_file, format, quality=88)
        else :
            image.save(thumbnail_file, format, quality=88)
        thumbnail_file.seek(0)
        return thumbnail_file, format.lower()
																														 
registerField(CutImageField,
              title='Cut Image',
              description='A field that cut images.')
